'''
synbiochem (c) University of Manchester 2015

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
# pylint: disable=too-many-arguments
import math
import random
import traceback
from synbiochem.utils.job import JobThread


class SimulatedAnnealer(JobThread):
    '''Class to perform simulated annealing method.'''

    def __init__(self, solution, acceptance=0.01, max_iter=10000,
                 heartbeat=1, verbose=False):
        self.__solution = solution
        self.__acceptance = acceptance
        self.__max_iter = max_iter
        self.__verbose = verbose
        self.__heartbeat = heartbeat
        JobThread.__init__(self)

    def run(self):
        '''Optimises a solution with simulated annealing.'''
        if self.__init():
            # Initialization:
            iteration = 0
            accepts = 0
            rejects = 0
            r_temp = 0.025
            cooling_rate = r_temp / 100

            energy = self.__solution.get_energy()

            while not self._cancelled \
                    and energy > self.__acceptance \
                    and iteration < self.__max_iter:
                iteration += 1
                energy_new = self.__solution.mutate()

                if energy_new < energy:
                    # Accept move immediately:
                    energy = energy_new
                    self.__accept(iteration)
                    print '\t'.join(['T', str(iteration), str(energy_new),
                                     str(self.__solution)])
                elif energy_new == energy:
                    # Reject move:
                    self.__solution.reject()
                    rejects += 1
                    print '\t'.join([' ', str(iteration), str(energy_new),
                                     str(self.__solution)])
                elif math.exp((energy - energy_new) / r_temp) > \
                        random.random():
                    # Accept move based on conditional probability:
                    energy = energy_new
                    self.__accept(iteration)
                    accepts += 1
                    print '\t'.join(['*', str(iteration), str(energy_new),
                                     str(self.__solution)])
                else:
                    # Reject move:
                    self.__solution.reject()
                    rejects += 1
                    print '\t'.join([' ', str(iteration), str(energy_new),
                                     str(self.__solution)])

                # Heartbeat:
                self.__check_heartbeat(iteration)

                # Simulated annealing control:
                if accepts + rejects > 50:
                    if float(accepts) / float(accepts + rejects) > 0.2:
                        # Too many accepts, reduce r_temp:
                        r_temp /= 2.0
                        accepts = 0
                        rejects = 0
                    elif float(accepts) / float(accepts + rejects) < 0.01:
                        # Too many rejects, increase r_temp:
                        r_temp *= 2.0
                        accepts = 0
                        rejects = 0

                r_temp *= 1 - cooling_rate

            if iteration == self.__max_iter:
                message = 'Unable to optimise in ' + str(self.__max_iter) + \
                    ' iterations'
                self.__fire_event('error', 100, iteration, message=message)
            elif self._cancelled:
                self.__fire_event('cancelled', 100, iteration,
                                  message='Job cancelled')
            else:
                self.__fire_event('finished', 100, iteration,
                                  message='Job completed')

    def __init(self):
        '''Initialise.'''
        self.__fire_event('running', 0, 0, message='Job initialising...')

        try:
            self.__solution.init()
        except ValueError:
            self.__fire_event('error', 100, 0, message=traceback.format_exc())
            return False

        self.__fire_event('running', 0, 0, message='Job initialised')
        return True

    def __check_heartbeat(self, iteration):
        '''Heartbeat.'''
        if float(iteration) % self.__heartbeat == 0:
            self.__fire_event('running',
                              float(iteration) / self.__max_iter * 100,
                              iteration,
                              'Running...')

    def __accept(self, iteration):
        '''Accept the current solution.'''
        self.__solution.accept()

        self.__fire_event('running', float(iteration) / self.__max_iter * 100,
                          iteration, 'Running...')

    def __fire_event(self, status, progress, iteration, message=''):
        '''Fires an event.'''
        event = {'update': {'status': status,
                            'message': message,
                            'progress': progress,
                            'iteration': iteration,
                            'max_iter': self.__max_iter,
                            'values': self.__solution.get_values()},
                 'query': self.__solution.get_query()
                 }

        if status == 'finished':
            event['result'] = self.__solution.get_result()

        self._fire_event(event)



'''
synbiochem (c) University of Manchester 2015

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
import random
import unittest

from synbiochem.utils import mut_utils, seq_utils


class MutationTest(unittest.TestCase):
    '''Test class for Mutation.'''

    def test_init_err_wt_seq_aa(self):
        '''Tests __init__.'''
        self.assertRaises(AssertionError, mut_utils.Mutation, 'J', 12, 'A')

    def test_init_err_mut_seq_aa(self):
        '''Tests __init__.'''
        self.assertRaises(AssertionError, mut_utils.Mutation, 'A', 12, 'J')

    def test_init_err_wt_seq_dna(self):
        '''Tests __init__.'''
        self.assertRaises(AssertionError, mut_utils.Mutation, 'D', 12, 'A',
                          'dna')

    def test_init_err_mut_seq_dna(self):
        '''Tests __init__.'''
        self.assertRaises(AssertionError, mut_utils.Mutation, 'A', 12, 'D',
                          'dna')

    def test_init_err_pos_zero(self):
        '''Tests __init__.'''
        self.assertRaises(AssertionError, mut_utils.Mutation, 'A', 0, 'A',
                          'dna')

    def test_init_err_pos_neg(self):
        '''Tests __init__.'''
        self.assertRaises(AssertionError, mut_utils.Mutation, 'A', -5, 'A',
                          'dna')

    def test_init_err_pos_non_int(self):
        '''Tests __init__.'''
        self.assertRaises(AssertionError, mut_utils.Mutation, 'A', 5.6, 'A',
                          'dna')

    def test_repr(self):
        '''Tests __repr__.'''
        mut = mut_utils.Mutation('A', 12, '-')
        self.assertEquals(mut.__repr__(), 'A12-')

    def test_str(self):
        '''Tests __str__.'''
        mut = mut_utils.Mutation('A', 12, 'V')
        self.assertEquals(mut.__str__(), 'A12V')

    def test_eq(self):
        '''Tests __eq__.'''
        mut_a = mut_utils.Mutation('A', 12, 'V')
        mut_b = mut_utils.Mutation('A', 12, 'V')
        mut_c = mut_utils.Mutation('A', 13, 'V')
        mut_d = mut_utils.Mutation('A', 12, 'T')

        self.assertEquals(mut_a, mut_b)
        self.assertNotEquals(mut_a, mut_c)
        self.assertNotEquals(mut_a, mut_d)

    def test_cmp(self):
        '''Tests __cmp__.'''
        mut_a = mut_utils.Mutation('A', 12, 'V')
        mut_b = mut_utils.Mutation('A', 5, 'V')
        mut_c = mut_utils.Mutation('A', 12, 'T')
        lst = [mut_a, mut_b, mut_c]
        sort_lst = [mut_b, mut_c, mut_a]

        self.assertNotEquals(lst, sorted(lst))
        self.assertEquals(sort_lst, sorted(lst))


class MutProbTest(unittest.TestCase):
    '''Test class for MutProb.'''

    def test_get_mut_prob(self):
        '''Tests get_mut_prob.'''
        aa_codes = dict(seq_utils.AA_CODES)
        aa_codes.pop('Stop')

        mut_prob = mut_utils.MutProbs()

        for _ in range(128):
            wt_res = random.choice(aa_codes.values())
            mut_res = random.choice(aa_codes.values())
            self.assertTrue(mut_prob.get_mut_prob(wt_res, mut_res) > 0)


class Test(unittest.TestCase):
    '''Test class for sequence_utils.'''

    def test_parse_mut_str(self):
        '''Tests parse_mut_str method.'''
        mut_a = mut_utils.Mutation('A', 12, 'V')
        mut_b = mut_utils.Mutation('P', 5, '-')
        mut_lst = [mut_a, mut_b]
        mut_str = ' '.join([str(mut) for mut in mut_lst])

        self.assertEquals(mut_str, 'A12V P5-')
        self.assertEquals(mut_utils.parse_mut_str(mut_str), mut_lst)

    def test_get_mutations(self):
        '''Tests get_mutations method.'''
        wt_seq = 'QWERTY'
        mt_seq = 'QTER-Y'
        mut_a = mut_utils.Mutation('W', 2, 'T')
        mut_b = mut_utils.Mutation('T', 5, '-')
        mut_lst = [mut_a, mut_b]
        self.assertEquals(mut_utils.get_mutations(wt_seq, mt_seq), mut_lst)

    def test_apply_mutations(self):
        '''Tests apply_mutations method.'''
        aa_len = 1024
        wt_seq = [random.choice(seq_utils.AA_CODES.values())
                  for _ in range(aa_len)]
        mt_seq = list(wt_seq)

        mutations = []

        for _ in range(16):
            pos = random.randint(0, aa_len)
            wt_res = wt_seq[pos]
            mt_res = random.choice(seq_utils.AA_CODES.values())
            mt_seq[pos] = mt_res
            mutations.append(mut_utils.Mutation(wt_res, pos + 1, mt_res))

        self.assertEquals(mut_utils.apply_mutations(wt_seq, mutations),
                          ''.join(mt_seq))


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
synbiochem (c) University of Manchester 2015

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  alanwilliams
'''
# pylint: disable=missing-docstring
# pylint: disable=too-few-public-methods
import json

import requests


_API_KEY = 'X-LC-APP-Auth'
_CHARSET = 'X-LC-APP-Charset'
_CHARSET_VALUE = 'UTF-8'


class LabCollectorClient(object):
    '''Class representing a LabCollector client.'''

    def __init__(self, url, apikey):
        self.__url = url + '/webservice/v1'
        self.__headers = {'Accept': 'application/json',
                          'Content-Type': 'multipart/form-data',
                          _API_KEY: apikey,
                          _CHARSET: _CHARSET_VALUE}

    @property
    def animals(self):
        response = requests.get(
            self.__url + '/animals', headers=self.__headers)
        return json.loads(response.content)



'''
synbiochem (c) University of Manchester 2015

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
import requests


def get(url, headers=None, verify=True):
    '''GETs url.'''
    return __process_resp(requests.get(url, headers=headers,
                                       verify=verify))


def put(url, data, headers, verify=True):
    '''POSTs data to url.'''
    return __process_resp(requests.put(url, data=data,
                                       headers=headers, verify=verify))


def post(url, data, headers, verify=True):
    '''POSTs data to url.'''
    return __process_resp(requests.post(url, data=data, headers=headers,
                                        verify=verify))


def delete(url, data=None, headers=None, verify=True):
    '''DELETEs data from url.'''
    return __process_resp(requests.delete(url, data=data, headers=headers,
                                          verify=verify))


def post_file(url, data, headers, verify=True):
    '''POSTs files to url.'''
    return __process_resp(requests.post(url, files=data, headers=headers,
                                        verify=verify))


def __process_resp(response):
    '''Processes a HTTP response.'''
    if response.status_code == 200:
        return response.text

    raise NetworkError(response.status_code, response.text)


class NetworkError(RuntimeError):
    '''Class to represent a network error.'''

    def __init__(self, status, text):
        super(NetworkError, self).__init__(NetworkError)
        self.__status = status
        self.__text = text

    def get_status(self):
        '''Gets the status.'''
        return self.__status

    def get_text(self):
        '''Gets the text.'''
        return self.__text

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return str({'status': self.__status, 'text': self.__text})



'''
synbiochem (c) University of Manchester 2015

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
# pylint: disable=no-member
# pylint: disable=too-many-arguments
import copy
from itertools import product
import math
import re
import uuid

from Bio.Restriction import RestrictionBatch
from Bio.Seq import Seq


SO_PROM = 'http://purl.obolibrary.org/obo/SO_0000167'
SO_RBS = 'http://purl.obolibrary.org/obo/SO_0000139'
SO_CDS = 'http://purl.obolibrary.org/obo/SO_0000316'
SO_PART = 'http://purl.obolibrary.org/obo/SO_0000804'
SO_ASS_COMP = 'http://purl.obolibrary.org/obo/SO_0000143'
SO_PLASMID = 'http://purl.obolibrary.org/obo/SO_0000637'
SO_DESIGNED = 'http://purl.obolibrary.org/obo/SO_0000546'
SO_RANDOM = 'http://purl.obolibrary.org/obo/SO_0000449'

_RDF_NS = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
_NS = {'ns': 'http://sbols.org/v1#',
       'rdf': _RDF_NS}


class DNA(dict):
    '''Class to represent a DNA object.'''

    def __init__(self, disp_id=None, name=None, desc=None, typ=None, seq=None,
                 start=1, end=float('NaN'), forward=True,
                 features=None, children=None, options=None,
                 links=None, parameters=None, temp_params=None):

        if seq is None:
            seq = ''

        dict.__init__(self)

        self.update({'disp_id': disp_id
                     if disp_id is not None else str(uuid.uuid4()),
                     'seq': re.sub(r'[\s]', '', seq.upper()),
                     'name': name,
                     'desc': desc,
                     'typ': typ,
                     'start': start,
                     'end': end if not math.isnan(end)
                     else start + len(re.sub(r'[\s]', '', seq.upper())) - 1,
                     'forward': forward,
                     'features': [] if features is None else features,
                     'children': [] if children is None else children,
                     'options': [] if options is None else options,
                     'links': [] if links is None else links,
                     'parameters': {} if parameters is None else parameters,
                     'temp_params': {} if temp_params is None else temp_params,
                     })

    def set_seq(self, seq):
        '''Sets sequence.'''
        self['seq'] = re.sub(r'[\s]', '', seq.upper())
        self['end'] = self['start'] + len(seq) - 1

    def copy(self):
        '''Copies the DNA object (making a deep copy).'''
        copy_dict = copy.deepcopy(self)
        copy_dna = DNA(**copy_dict)
        return copy_dna

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __repr__(self):
        return self['name']


def get_dna(dct):
    '''Factory method for constructing DNA object from dict.'''
    dna = DNA(**dct)
    dna['features'] = [get_dna(feat) for feat in dna['features']]
    dna['children'] = [get_dna(child) for child in dna['children']]
    dna['options'] = [get_dna(opt) for opt in dna['options']]
    return dna


def concat(dnas):
    '''Concatenates a list of DNA objects into a single DNA object.'''
    concat_dna = dnas[0].copy()
    concat_dna['disp_id'] = str(uuid.uuid4())

    for dna in dnas[1:]:
        concat_dna = add(concat_dna, dna)

    return concat_dna


def apply_restricts(dna, restricts, circular=False):
    '''Applies restriction site cleavage to forward and reverse strands.'''
    out_dnas = [dna]

    for restrict in restricts:
        batch = RestrictionBatch()
        batch.add(str(restrict))
        restrict = batch.get(str(restrict))
        out_dnas = _apply_restrict_to_dnas(out_dnas, restrict, circular)

    return out_dnas


def add(dna1, dna2):
    '''Adds two DNA objects together.'''
    # Add names, etc.
    dna1.disp_id = str(uuid.uuid4())
    dna1['name'] = _concat([dna1['name'], dna2['name']])
    dna1['desc'] = _concat([dna1['desc'], dna2['desc']])

    # Add sequences:
    orig_seq_len = len(dna1['seq'])
    dna1['seq'] += dna2['seq']
    dna1['end'] = len(dna1['seq'])

    # Update parameters:
    for key, value in dna2['parameters'].iteritems():
        param = dna1['parameters'].get(key, None)

        if param is None:
            dna1['parameters'][key] = value
        elif isinstance(param, list):
            param.append(value)
            dna1['parameters'][key] = param
        else:
            dna1['parameters'][key] = [param, value]

    # Update features:
    for feature in dna2.get('features', []):
        feature = feature.copy()
        feature['start'] += orig_seq_len
        feature['end'] += orig_seq_len
        dna1['features'].append(feature)

    return dna1


def expand(dna):
    '''Expand a DNA object containing options into distinct DNA objects.'''
    features = dna.pop('features')
    dna['features'] = []

    dnas = [dna]

    for feature in features:
        if feature['options']:
            additions = feature['options']
        elif feature['seq']:
            additions = [feature]
        else:
            continue

        for addition in additions:
            addition['features'].append(addition.copy())

        dnas = [add(dna.copy(), addition)
                for dna, addition in product(dnas, additions)
                if len(addition['seq'])]

    return dnas


def _concat(strs):
    '''Concatenates non-None strings.'''
    return ' - '.join([string for string in strs
                       if string is not None and len(string)])


def _apply_restrict_to_dnas(dnas, restrict, circular):
    '''Applies restriction site cleavage to forward and reverse strands.'''
    out_dnas = []

    for dna in dnas:
        restrict.catalyse(Seq(dna['seq']), linear=not circular)
        start = 0
        seqs = []

        for result in restrict.results:
            end = result - 1
            seqs.append((dna['seq'][start:end], start, end))
            start = end

        seqs.append((dna['seq'][start:], start, len(dna['seq'])))

        if circular:
            seqs = [(seqs[-1][0] + seqs[0][0], seqs[-1][1], seqs[0][2])] + \
                seqs[1:-1]

        for seq in seqs:
            out_dnas.append(_get_concat_dna(dna, seq[0], seq[1], seq[2]))

    return out_dnas


def _apply_restrict_to_seq(seq, restrict):
    '''Applies restriction site cleavage to a sequence.'''
    sub_seqs = [(match.group(0), match.start())
                for match in re.finditer(restrict, seq)]
    end = sub_seqs[0][1] if sub_seqs else len(seq)
    return [(seq[:end], 0)] + sub_seqs


def _get_concat_dna(parent_dna, seq, start, end):
    '''Returns a DNA object from the supplied subsequence from a parent DNA
    object.'''
    if parent_dna['seq'] == seq:
        disp_id = parent_dna['disp_id']
        frag_str = ''
    else:
        disp_id = str(uuid.uuid4())
        frag_str = ' [' + str(start) + ':' + str(end) + ']'

    dna = DNA(disp_id=disp_id,
              name=parent_dna['name'] + frag_str,
              desc=parent_dna['desc'] + frag_str,
              seq=seq)

    for feature in parent_dna['features']:
        if feature['start'] >= start and feature['end'] <= end:
            copy_feature = feature.copy()
            copy_feature['start'] -= start
            copy_feature['end'] -= start
            dna['features'].append(copy_feature)

    return dna



'''
synbiochem (c) University of Manchester 2015

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
# pylint: disable=too-many-public-methods
import unittest

import synbiochem.utils.chem_utils as chm_util


class Test(unittest.TestCase):
    '''Test class for chemical_utils.'''

    def test_get_molecular_mass(self):
        '''Tests get_molecular_mass method.'''
        self.assertAlmostEqual(chm_util.get_molecular_mass('H2O'),
                               18.010564684)

    def test_get_elem_comp(self):
        '''Tests get_elem_comp method.'''
        self.assertEqual(chm_util.get_elem_comp('Fe4S'), {'Fe': 4, 'S': 1})

    def test_parse_equation(self):
        '''Tests parse_equation method.'''
        eqn = '5.6 Fe4S + -3.2 water = 17.8 SiO2'
        self.assertEqual(chm_util.parse_equation(eqn),
                         [['Fe4S', -5.6], ['water', 3.2], ['SiO2', 17.8]])

    def test_parse_equation_error(self):
        '''Tests parse_equation method (with error).'''
        eqn = '5.6 Fe4S + -3.2 water = n+m SiO2'
        self.assertRaises(ValueError, chm_util.parse_equation, eqn)


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
SYNBIOCHEM-DB (c) University of Manchester 2017

SYNBIOCHEM-DB is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
import csv
import os
import sys

import xlrd


def convert(xl_filename):
    '''Convert Excel file.'''
    dir_name, _ = os.path.splitext(xl_filename)

    if not os.path.exists(dir_name):
        os.makedirs(dir_name)

    workbook = xlrd.open_workbook(xl_filename)

    for sheet in workbook.sheets():
        csv_filename = os.path.join(dir_name, sheet.name + '.csv')

        with open(csv_filename, 'wb') as csv_file:
            writer = csv.writer(csv_file)

            for rownum in xrange(sheet.nrows):
                writer.writerow(sheet.row_values(rownum))

            csv_file.close()

    return dir_name


def main(args):
    '''main method.'''
    print convert(args[0])


if __name__ == '__main__':
    main(sys.argv[1:])



'''
synbiochem (c) University of Manchester 2015

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''


def pairwise(iterable):
    '''s -> (s0,s1), (s1,s2), (s2, s3), ...'''
    return [(iterable[i], iterable[i + 1]) for i in range(len(iterable) - 1)]



'''
synbiochem (c) University of Manchester 2017

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
# pylint: disable=broad-except
from Queue import Queue
from threading import Thread


class Worker(Thread):
    '''Thread executing tasks from a given tasks queue.'''

    def __init__(self, tasks):
        Thread.__init__(self)
        self.__tasks = tasks
        self.daemon = True
        self.start()

    def run(self):
        while True:
            func, args, kargs = self.__tasks.get()
            try:
                func(*args, **kargs)
            except Exception, err:
                print err
            finally:
                # Mark this task as done, whether an exception happened or not
                self.__tasks.task_done()


class ThreadPool(object):
    '''Pool of threads consuming tasks from a queue.'''

    def __init__(self, num_threads):
        self.__tasks = Queue(num_threads)

        for _ in range(num_threads):
            Worker(self.__tasks)

    def add_task(self, func, *args, **kargs):
        '''Add a task to the queue.'''
        self.__tasks.put((func, args, kargs))

    def map(self, func, args_list):
        '''Add a list of tasks to the queue.'''
        for args in args_list:
            self.add_task(func, args)

    def wait_completion(self):
        '''Wait for completion of all the tasks in the queue.'''
        self.__tasks.join()



'''
SYNBIOCHEM-DB (c) University of Manchester 2015

SYNBIOCHEM-DB is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''



'''
SYNBIOCHEM-DB (c) University of Manchester 2015

SYNBIOCHEM-DB is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
# pylint: disable=too-many-public-methods
import unittest

from synbiochem.biochem4j import taxonomy


class TestTaxonomy(unittest.TestCase):
    '''Test class for taxonomy.'''

    def test_get_synonyms_by_id(self):
        '''Tests get_synonyms_by_id method.'''
        synonyms = taxonomy.get_synonyms_by_id('511145')
        self.assertTrue('Escherichia coli strain MG1655' in synonyms)

    def test_get_children_by_id(self):
        '''Tests get_children_by_id method.'''
        children = taxonomy.get_children_by_id('83333')
        self.assertTrue('511145' in [child['taxonomy'] for child in children])

    def test_get_parent_by_id(self):
        '''Tests get_parent_by_id method.'''
        self.assertEquals(taxonomy.get_parent_by_id('511145')['taxonomy'],
                          '83333')

    def test_get_parent_by_name(self):
        '''Tests taxonomy method.'''
        name = 'Escherichia coli str. K-12 substr. MG1655'

        self.assertEquals(taxonomy.get_parent_by_name(name)['taxonomy'],
                          '83333')


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
synbiochem (c) University of Manchester 2017

synbiochem is licensed under the MIT License.

To view a copy of this license, visit <http://opensource.org/licenses/MIT/>.

@author:  neilswainston
'''
# pylint: disable=too-many-arguments
import os
import shutil
import subprocess


def create_db(neo4j_root_loc, nodes_files, rels_files, db_name='graph.db',
              ignore_duplicates=False, delimiter=',', array_delimiter=';'):
    '''Creates the database from csv files.

    Import format is:

    neo4j-admin import --nodes NODES_FILE --relationships RELS_FILE

    '''
    # Stop database:
    bin_loc = os.path.join(neo4j_root_loc, 'bin')
    subprocess.call([os.path.join(bin_loc, 'neo4j'), 'stop'])

    # Remove database directory if it already exists:
    db_loc = os.path.join(neo4j_root_loc, 'data', 'databases', db_name)

    if os.path.exists(db_loc):
        shutil.rmtree(db_loc)

    # Import database:
    admin_loc = os.path.join(bin_loc, 'neo4j-admin')

    params = [admin_loc,
              'import',
              '--database', db_name]

    for node_file in nodes_files:
        params.extend(['--nodes', node_file])

    for rels_file in rels_files:
        params.extend(['--relationships', rels_file])

    if ignore_duplicates:
        params.append('--ignore-duplicate-nodes')

    params.extend(['--delimiter', delimiter])
    params.extend(['--array-delimiter', array_delimiter])

    subprocess.call(params)
